BUint

Struct BUint 

pub struct BUint<const N: usize> { /* private fields */ }
Unsigned integer type composed of u64 digits, of arbitrary fixed size which must be known at compile time.

Digits are stored in little endian (least significant digit first). This integer type aims to exactly replicate the behaviours of Rust’s built-in unsigned integer types: u8, u16, u32, u64, u128 and usize. The const generic parameter N is the number of u64 digits that are stored.

BUint implements all the arithmetic traits from the core::ops module. The behaviour of the implementation of these traits is the same as for Rust’s primitive integers - i.e. in debug mode it panics on overflow, and in release mode it performs two’s complement wrapping (see https://doc.rust-lang.org/book/ch03-02-data-types.html#integer-overflow). However, an attempt to divide by zero or calculate a remainder with a divisor of zero will always panic, unless the checked_ methods are used, which never panic.

Implementations

Source

impl<const N: usize> BUint<N>

Bigint helper methods: common functions used to implement big integer arithmetic.

pub const fn carrying_add(self, rhs: Self, carry: bool) -> (Self, bool)

pub const fn borrowing_sub(self, rhs: Self, borrow: bool) -> (Self, bool)

pub const fn widening_mul(self, rhs: Self) -> (Self, Self)

pub const fn carrying_mul(self, rhs: Self, carry: Self) -> (Self, Self)

Source

impl<const N: usize> BUint<N>

Checked arithmetic methods which act on self: self.checked_.... Each method cannot panic and returns an Option<Self>. None is returned when overflow would have occurred or there was an attempt to divide by zero or calculate a remainder with a divisor of zero.

pub const fn checked_add(self, rhs: Self) -> Option<Self>

pub const fn checked_add_signed(self, rhs: BInt<N>) -> Option<Self>

pub const fn checked_sub(self, rhs: Self) -> Option<Self>

pub const fn checked_mul(self, rhs: Self) -> Option<Self>

pub const fn checked_div(self, rhs: Self) -> Option<Self>

pub const fn checked_div_euclid(self, rhs: Self) -> Option<Self>

pub const fn checked_rem(self, rhs: Self) -> Option<Self>

pub const fn checked_rem_euclid(self, rhs: Self) -> Option<Self>

pub const fn checked_neg(self) -> Option<Self>

pub const fn checked_shl(self, rhs: u32) -> Option<Self>

pub const fn checked_shr(self, rhs: u32) -> Option<Self>

pub const fn checked_pow(self, pow: u32) -> Option<Self>

pub const fn checked_next_multiple_of(self, rhs: Self) -> Option<Self>

pub const fn checked_ilog2(self) -> Option<u32>

pub const fn checked_ilog10(self) -> Option<u32>

pub const fn checked_ilog(self, base: Self) -> Option<u32>

pub const fn checked_next_power_of_two(self) -> Option<Self>

Returns the smallest power of two greater than or equal to self. If the next power of two is greater than Self::MAX, None is returned, otherwise the power of two is wrapped in Some.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.checked_next_power_of_two.

Examples
use bnum::types::U256;

let n = U256::from(2u8);
assert_eq!(n.checked_next_power_of_two(), Some(n));
let m = U256::from(3u8);
assert_eq!(U256::MAX.checked_next_power_of_two(), None);
Source

impl<const N: usize> BUint<N>

The latest version of the nightly Rust compiler at the time v0.7.0 was published (v1.71.0-nightly) unfortunately dropped support for the const implementation of common standard library traits such as Add, BitOr, etc. These methods below have therefore been provided to allow use of some of the methods these traits provided, in a const context.

pub const fn bitand(self, rhs: Self) -> Self

pub const fn bitor(self, rhs: Self) -> Self

pub const fn bitxor(self, rhs: Self) -> Self

pub const fn not(self) -> Self

pub const fn eq(&self, other: &Self) -> bool

pub const fn ne(&self, other: &Self) -> bool

pub const fn cmp(&self, other: &Self) -> Ordering

pub const fn max(self, other: Self) -> Self

pub const fn min(self, other: Self) -> Self

pub const fn clamp(self, min: Self, max: Self) -> Self

pub const fn lt(&self, other: &Self) -> bool

pub const fn le(&self, other: &Self) -> bool

pub const fn gt(&self, other: &Self) -> bool

pub const fn ge(&self, other: &Self) -> bool

pub const fn add(self, rhs: Self) -> Self

pub const fn mul(self, rhs: Self) -> Self

pub const fn shl(self, rhs: u32) -> Self

pub const fn shr(self, rhs: u32) -> Self

pub const fn sub(self, rhs: Self) -> Self

pub const fn div(self, rhs: Self) -> Self

pub const fn rem(self, rhs: Self) -> Self

Source

impl<const N: usize> BUint<N>

Associated constants for this type.

pub const MIN: Self

The minimum value that this type can represent.

Examples
use bnum::types::U512;

assert_eq!(!U512::MIN, U512::MAX);
pub const MAX: Self

The maximum value that this type can represent.

Examples
use bnum::types::U512;

assert_eq!(U512::MAX.wrapping_add(U512::ONE), U512::MIN);
pub const BITS: u32

The total number of bits that this type contains.

Examples
use bnum::types::U512;

assert_eq!(U512::BITS, 512);
pub const BYTES: u32

The total number of bytes that this type contains.

Examples
use bnum::types::U512;

assert_eq!(U512::BYTES, 512 / 8);
pub const ZERO: Self = Self::MIN

The value of 0 represented by this type.

Examples
use bnum::types::U512;

assert_eq!(U512::ZERO, U512::from(0u8));
pub const ONE: Self

The value of 1 represented by this type.

pub const TWO: Self

The value of 2 represented by this type.

pub const THREE: Self

The value of 3 represented by this type.

pub const FOUR: Self

The value of 4 represented by this type.

pub const FIVE: Self

The value of 5 represented by this type.

pub const SIX: Self

The value of 6 represented by this type.

pub const SEVEN: Self

The value of 7 represented by this type.

pub const EIGHT: Self

The value of 8 represented by this type.

pub const NINE: Self

The value of 9 represented by this type.

pub const TEN: Self

The value of 10 represented by this type.

Source

impl<const N: usize> BUint<N>

Methods which convert a BUint to and from data stored in different endianness.

pub const fn from_be(x: Self) -> Self

Converts an integer from big endian to the target’s endianness.

On big endian this is a no-op. On little endian the bytes are swapped.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.from_be.

pub const fn from_le(x: Self) -> Self

Converts an integer from little endian to the target’s endianness.

On little endian this is a no-op. On big endian the bytes are swapped.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.from_le.

pub const fn to_be(self) -> Self

Converts self from big endian to the target’s endianness.

On big endian this is a no-op. On little endian the bytes are swapped.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.to_be.

pub const fn to_le(self) -> Self

Converts self from little endian to the target’s endianness.

On little endian this is a no-op. On big endian the bytes are swapped.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.to_le.

pub const fn from_be_slice(slice: &[u8]) -> Option<Self>

Create an integer value from a slice of bytes in big endian. The value is wrapped in an Option as the integer represented by the slice of bytes may represent an integer too large to be represented by the type.

If the length of the slice is shorter than Self::BYTES, the slice is padded with zeros at the start so that it’s length equals Self::BYTES.

If the length of the slice is longer than Self::BYTES, None will be returned, unless leading zeros from the slice can be removed until the length of the slice equals Self::BYTES.

Examples
use bnum::types::U128;

let value_from_array = U128::from(u128::from_be_bytes([0, 0, 0x56, 0x78, 0x90, 0x12, 0x34, 0x56, 0x78, 0x90, 0x12, 0x34, 0x56, 0x78, 0x90, 0x12]));
// using the `from_be_bytes` method on the primitve `u128` here instead of on `U128` as `from_be_bytes` is currently only available in bnum on nightly
let value_from_slice = U128::from_be_slice(&[0x56, 0x78, 0x90, 0x12, 0x34, 0x56, 0x78, 0x90, 0x12, 0x34, 0x56, 0x78, 0x90, 0x12]).unwrap();
let value_from_long_slice = U128::from_be_slice(&[0, 0, 0, 0, 0, 0x56, 0x78, 0x90, 0x12, 0x34, 0x56, 0x78, 0x90, 0x12, 0x34, 0x56, 0x78, 0x90, 0x12]).unwrap();

assert_eq!(value_from_array, value_from_slice);
assert_eq!(value_from_array, value_from_long_slice);

let invalid_slice = &[0x56, 0x78, 0x90, 0x12, 0x34, 0x56, 0x78, 0x90, 0x12, 0x34, 0x56, 0x78, 0x90, 0x12, 0x34, 0x56, 0x78, 0x90];
assert_eq!(U128::from_be_slice(invalid_slice), None);
pub const fn from_le_slice(slice: &[u8]) -> Option<Self>

Creates an integer value from a slice of bytes in little endian. The value is wrapped in an Option as the bytes may represent an integer too large to be represented by the type.

If the length of the slice is shorter than Self::BYTES, the slice is padded with zeros at the end so that it’s length equals Self::BYTES.

If the length of the slice is longer than Self::BYTES, None will be returned, unless trailing zeros from the slice can be removed until the length of the slice equals Self::BYTES.

Examples
use bnum::types::U128;

let value_from_array = U128::from(u128::from_le_bytes([0x12, 0x90, 0x78, 0x56, 0x34, 0x12, 0x90, 0x78, 0x56, 0x34, 0x12, 0x90, 0x78, 0x56, 0, 0]));
// using the `from_le_bytes` method on the primitve `u128` here instead of on `U128` as `from_le_bytes` is currently only available in bnum on nightly
let value_from_slice = U128::from_le_slice(&[0x12, 0x90, 0x78, 0x56, 0x34, 0x12, 0x90, 0x78, 0x56, 0x34, 0x12, 0x90, 0x78, 0x56]).unwrap();
let value_from_long_slice = U128::from_le_slice(&[0x12, 0x90, 0x78, 0x56, 0x34, 0x12, 0x90, 0x78, 0x56, 0x34, 0x12, 0x90, 0x78, 0x56, 0, 0, 0, 0, 0, 0]).unwrap();

assert_eq!(value_from_array, value_from_slice);
assert_eq!(value_from_array, value_from_long_slice);

let invalid_slice = &[0x56, 0x78, 0x90, 0x12, 0x34, 0x56, 0x78, 0x90, 0x12, 0x34, 0x56, 0x78, 0x90, 0x12, 0x34, 0x56, 0x78, 0x90];
assert_eq!(U128::from_le_slice(invalid_slice), None);
Source

impl<const N: usize> BUint<N>

Overflowing arithmetic methods which act on self: self.overflowing_.... Each method returns a tuple of type (Self, bool) where the first item of the tuple is the result of the calculation truncated to the number of bits of self, and the second item is a boolean which indicates whether overflow occurred (i.e. if the number of bits of the result of the calculation exceeded the number of bits of self).

pub const fn overflowing_add(self, rhs: Self) -> (Self, bool)

pub const fn overflowing_add_signed(self, rhs: BInt<N>) -> (Self, bool)

pub const fn overflowing_sub(self, rhs: Self) -> (Self, bool)

pub const fn overflowing_mul(self, rhs: Self) -> (Self, bool)

pub const fn overflowing_div(self, rhs: Self) -> (Self, bool)

pub const fn overflowing_div_euclid(self, rhs: Self) -> (Self, bool)

pub const fn overflowing_rem(self, rhs: Self) -> (Self, bool)

pub const fn overflowing_rem_euclid(self, rhs: Self) -> (Self, bool)

pub const fn overflowing_neg(self) -> (Self, bool)

pub const fn overflowing_shl(self, rhs: u32) -> (Self, bool)

pub const fn overflowing_shr(self, rhs: u32) -> (Self, bool)

pub const fn overflowing_pow(self, pow: u32) -> (Self, bool)

Source

impl<const N: usize> BUint<N>

Methods which convert a BUint between different types in a given radix (base).

pub const fn parse_bytes(buf: &[u8], radix: u32) -> Option<Self>

Converts a byte slice in a given base to an integer. The input slice must contain ascii/utf8 characters in [0-9a-zA-Z].

This function is equivalent to the from_str_radix function for a string slice equivalent to the byte slice and the same radix.

Returns None if the conversion of the byte slice to string slice fails or if a digit is larger than or equal to the given radix, otherwise the integer is wrapped in Some.

Panics

This function panics if radix is not in the range from 2 to 36 inclusive.

Examples
use bnum::types::U512;

let src = "394857hdgfjhsnkg947dgfjkeita";
assert_eq!(U512::from_str_radix(src, 32).ok(), U512::parse_bytes(src.as_bytes(), 32));
pub const fn from_radix_be(buf: &[u8], radix: u32) -> Option<Self>

Converts a slice of big-endian digits in the given radix to an integer. Each u8 of the slice is interpreted as one digit of base radix of the number, so this function will return None if any digit is greater than or equal to radix, otherwise the integer is wrapped in Some.

Panics

This function panics if radix is not in the range from 2 to 256 inclusive.

Examples
use bnum::types::U512;

let n = U512::from(34598748526857897975u128);
let digits = n.to_radix_be(12);
assert_eq!(Some(n), U512::from_radix_be(&digits, 12));
pub const fn from_radix_le(buf: &[u8], radix: u32) -> Option<Self>

Converts a slice of little-endian digits in the given radix to an integer. Each u8 of the slice is interpreted as one digit of base radix of the number, so this function will return None if any digit is greater than or equal to radix, otherwise the integer is wrapped in Some.

Panics

This function panics if radix is not in the range from 2 to 256 inclusive.

Examples
use bnum::types::U512;

let n = U512::from(109837459878951038945798u128);
let digits = n.to_radix_le(15);
assert_eq!(Some(n), U512::from_radix_le(&digits, 15));
pub const fn from_str_radix( src: &str, radix: u32, ) -> Result<Self, ParseIntError>

Converts a string slice in a given base to an integer.

The string is expected to be an optional + sign followed by digits. Leading and trailing whitespace represent an error. Digits are a subset of these characters, depending on radix:

  • 0-9
  • a-z
  • A-Z
Panics

This function panics if radix is not in the range from 2 to 36 inclusive.

Examples
use bnum::types::U512;

assert_eq!(U512::from_str_radix("A", 16), Ok(U512::from(10u128)));
pub const fn parse_str_radix(src: &str, radix: u32) -> Self

This function works the same as from_str_radix except that it returns a BUint instead of a Result<BUint, ParseIntError>.

Panics

This function panics if radix is not in the range from 2 to 36 inclusive, or if the string cannot be parsed successfully.

Examples

Basic usage:

// The below example will fail to compile, as the function will panic at compile time:
use bnum::types::U256;

const DOESNT_COMPILE: U256 = U256::parse_str_radix("a13423", 10);
// Gives a compile error of "error[E0080]: evaluation of constant value failed... the evaluated program panicked at 'attempt to parse integer from string containing invalid digit'", 
pub fn to_str_radix(&self, radix: u32) -> String

Returns the integer as a string in the given radix.

Panics

This function panics if radix is not in the range from 2 to 36 inclusive.

Examples
use bnum::types::U512;

let src = "934857djkfghhkdfgbf9345hdfkh";
let n = U512::from_str_radix(src, 36).unwrap();
assert_eq!(n.to_str_radix(36), src);
pub fn to_radix_be(&self, radix: u32) -> Vec<u8>

Returns the integer in the given base in big-endian digit order.

Panics

This function panics if radix is not in the range from 2 to 256 inclusive.

use bnum::types::U512;

let digits = &[3, 55, 60, 100, 5, 0, 5, 88];
let n = U512::from_radix_be(digits, 120).unwrap();
assert_eq!(n.to_radix_be(120), digits);
pub fn to_radix_le(&self, radix: u32) -> Vec<u8>

Returns the integer in the given base in little-endian digit order.

Panics

This function panics if radix is not in the range from 2 to 256 inclusive.

use bnum::types::U512;

let digits = &[1, 67, 88, 200, 55, 68, 87, 120, 178];
let n = U512::from_radix_le(digits, 250).unwrap();
assert_eq!(n.to_radix_le(250), digits);
Source

impl<const N: usize> BUint<N>

Saturating arithmetic methods which act on self: self.saturating_.... For each method, if overflow or underflow occurs, the largest or smallest value that can be represented by Self is returned instead.

Source

impl<const N: usize> BUint<N>

Strict arithmetic methods which act on self: self.strict_.... Each method will always panic if overflow/underflow occurs (i.e. when the checked equivalent would return None), regardless of whether overflow checks are enabled.

pub const fn strict_add(self, rhs: Self) -> Self

pub const fn strict_sub(self, rhs: Self) -> Self

pub const fn strict_mul(self, rhs: Self) -> Self

pub const fn strict_div(self, rhs: Self) -> Self

pub const fn strict_div_euclid(self, rhs: Self) -> Self

pub const fn strict_rem(self, rhs: Self) -> Self

pub const fn strict_rem_euclid(self, rhs: Self) -> Self

pub const fn strict_neg(self) -> Self

pub const fn strict_shl(self, rhs: u32) -> Self

pub const fn strict_shr(self, rhs: u32) -> Self

pub const fn strict_pow(self, exp: u32) -> Self

pub const fn strict_add_signed(self, rhs: BInt<N>) -> Self

Source

impl<const N: usize> BUint<N>

Unchecked arithmetic methods which act on self: self.unchecked_.... Each method results in undefined behavior if overflow/underflow occurs (i.e. when the checked equivalent would return None).

Source

impl<const N: usize> BUint<N>

Wrapping arithmetic methods which act on self: self.wrapping_.... Each method returns of the calculation truncated to the number of bits of self (i.e. they each return the first item in the tuple returned by their overflowing equivalent).

pub const fn wrapping_add(self, rhs: Self) -> Self

pub const fn wrapping_add_signed(self, rhs: BInt<N>) -> Self

pub const fn wrapping_sub(self, rhs: Self) -> Self

pub const fn wrapping_mul(self, rhs: Self) -> Self

pub const fn wrapping_div(self, rhs: Self) -> Self

pub const fn wrapping_div_euclid(self, rhs: Self) -> Self

pub const fn wrapping_rem(self, rhs: Self) -> Self

pub const fn wrapping_rem_euclid(self, rhs: Self) -> Self

pub const fn wrapping_neg(self) -> Self

pub const fn wrapping_shl(self, rhs: u32) -> Self

pub const fn wrapping_shr(self, rhs: u32) -> Self

pub const fn wrapping_pow(self, pow: u32) -> Self

pub const fn wrapping_next_power_of_two(self) -> Self

Returns the smallest power of two greater than or equal to self. If the next power of two is greater than Self::MAX, the return value is wrapped to Self::MIN.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.wrapping_next_power_of_two.

Examples
use bnum::types::U256;

let n = U256::from(31u8);
assert_eq!(n.wrapping_next_power_of_two(), 32u8.into());
assert_eq!(U256::MAX.wrapping_next_power_of_two(), U256::MIN);
Source

impl<const N: usize> BUint<N>

pub const fn count_ones(self) -> u32

Returns the number of ones in the binary representation of self.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.count_ones.

Examples
use bnum::types::U1024;

let n = U1024::from(0b010111101010000u16);
assert_eq!(n.count_ones(), 7);
pub const fn count_zeros(self) -> u32

Returns the number of zeros in the binary representation of self.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.count_zeros.

Examples
use bnum::types::U1024;

assert_eq!((!U1024::ZERO).count_zeros(), 0);
pub const fn leading_zeros(self) -> u32

Returns the number of leading zeros in the binary representation of self.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.leading_zeros.

Examples
use bnum::types::U1024;

let n = U1024::ZERO;
assert_eq!(n.leading_zeros(), 1024);
pub const fn trailing_zeros(self) -> u32

Returns the number of trailing zeros in the binary representation of self.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.trailing_zeros.

Examples
use bnum::types::U1024;

let n = (!U1024::ZERO) << 6u32;
assert_eq!(n.trailing_zeros(), 6);
pub const fn leading_ones(self) -> u32

Returns the number of leading ones in the binary representation of self.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.leading_ones.

Examples
use bnum::types::U1024;

let n = !U1024::ZERO;
assert_eq!(n.leading_ones(), 1024);
pub const fn trailing_ones(self) -> u32

Returns the number of trailing ones in the binary representation of self.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.trailing_ones.

Examples
use bnum::types::U1024;

let n = U1024::from(0b1000101011011u16);
assert_eq!(n.trailing_ones(), 2);
pub const fn cast_signed(self) -> BInt<N>

pub const fn rotate_left(self, n: u32) -> Self

Shifts the bits to the left by a specified amount, n, wrapping the truncated bits to the end of the resulting integer.

Please note this isn’t the same operation as the << shifting operator!

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.rotate_left.

pub const fn rotate_right(self, n: u32) -> Self

Shifts the bits to the left by a specified amount, n, wrapping the truncated bits to the end of the resulting integer.

Please note this isn’t the same operation as the >> shifting operator!

self.rotate_right(n) is equivalent to self.rotate_left(Self::BITS - n).

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.rotate_right.

pub const fn swap_bytes(self) -> Self

Reverses the byte order of the integer.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.swap_bytes.

Examples
use bnum::types::U256;

let n = U256::from(0x12345678901234567890123456789012u128);
assert_eq!(n.swap_bytes().swap_bytes(), n);
pub const fn reverse_bits(self) -> Self

Reverses the order of bits in the integer. The least significant bit becomes the most significant bit, second least-significant bit becomes second most-significant bit, etc.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.reverse_bits.

Examples
use bnum::types::U256;

let n = U256::from(0x12345678901234567890123456789012u128);
assert_eq!(n.reverse_bits().reverse_bits(), n);
pub const fn pow(self, exp: u32) -> Self

Raises self to the power of exp, using exponentiation by squaring.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.pow.

Examples
use bnum::types::U256;

let n = U256::from(3u8);
assert_eq!(n.pow(5), (243u8).into());
pub const fn div_euclid(self, rhs: Self) -> Self

pub const fn rem_euclid(self, rhs: Self) -> Self

pub const fn is_power_of_two(self) -> bool

Returns true if and only if self == 2^k for some integer k.

Examples
use bnum::types::U256;

let n = U256::from(1u16 << 14);
assert!(n.is_power_of_two());
let m = U256::from(100u8);
assert!(!m.is_power_of_two());
pub const fn next_power_of_two(self) -> Self

When return value overflows, it panics in debug mode and the return value is wrapped to 0 in release mode (the only situation in which method can return 0).

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.next_power_of_two.

Examples
use bnum::types::U256;

let n = U256::from(2u8);
assert_eq!(n.next_power_of_two(), n);
assert_eq!(U256::from(3u8).next_power_of_two(), 4u8.into());
assert_eq!(U256::ZERO.next_power_of_two(), U256::ONE);
pub const fn midpoint(self, rhs: Self) -> Self

pub const fn ilog2(self) -> u32

pub const fn ilog10(self) -> u32

pub const fn ilog(self, base: Self) -> u32

pub const fn abs_diff(self, other: Self) -> Self

pub const fn next_multiple_of(self, rhs: Self) -> Self

pub const fn div_floor(self, rhs: Self) -> Self

pub const fn div_ceil(self, rhs: Self) -> Self

Source

impl<const N: usize> BUint<N>

pub const fn bits(&self) -> u32

Returns the smallest number of bits necessary to represent self.

This is equal to the size of the type in bits minus the leading zeros of self.

Examples
use bnum::types::U256;

assert_eq!(U256::from(0b1111001010100u16).bits(), 13);
assert_eq!(U256::ZERO.bits(), 0);
pub const fn bit(&self, index: u32) -> bool

Returns a boolean representing the bit in the given position (true if the bit is set). The least significant bit is at index 0, the most significant bit is at index Self::BITS - 1.

Examples
use bnum::types::U256;

let n = U256::from(0b001010100101010101u32);
assert!(n.bit(0));
assert!(!n.bit(1));
assert!(!n.bit(U256::BITS - 1));
pub fn set_bit(&mut self, index: u32, value: bool)

Sets/unsets the bit in the given position (1 if value is true). The least significant bit is at index 0, the most significant bit is at index Self::BITS - 1.

Examples
use bnum::types::U256;

let mut n = U256::from(0b001010100101010101u32);
assert!(n.bit(2));
n.set_bit(2, false);
assert!(!n.bit(2));
assert_eq!(n, U256::from(0b001010100101010001u32));
pub const fn power_of_two(power: u32) -> Self

Returns an integer whose value is 2^power. This is faster than using a shift left on Self::ONE.

Panics

This function will panic if power is greater than or equal to Self::BITS.

Examples
use bnum::types::U256;

let power = 11;
assert_eq!(U256::power_of_two(11), (1u128 << 11).into());
pub const fn digits(&self) -> &[u64; N]

Returns the digits stored in self as an array. Digits are little endian (least significant digit first).

pub fn digits_mut(&mut self) -> &mut [u64; N]

Returns the digits stored in self as a mutable array. Digits are little endian (least significant digit first).

pub const fn from_digits(digits: [u64; N]) -> Self

Creates a new unsigned integer from the given array of digits. Digits are stored as little endian (least significant digit first).

pub const fn from_digit(digit: u64) -> Self

Creates a new unsigned integer from the given digit. The given digit is stored as the least significant digit.

pub const fn is_zero(&self) -> bool

Returns whether self is zero.

Examples
use bnum::types::U256;

assert!(U256::ZERO.is_zero());
assert!(!U256::ONE.is_zero());
pub const fn is_one(&self) -> bool

Returns whether self is one.

Examples
use bnum::types::U256;

assert!(U256::ONE.is_one());
assert!(!U256::MAX.is_one());